import requests
from datetime import datetime
import json

insertUrl = 'http://10.220.6.138/api/khgz/project'
insertTestUrl = 'http://10.220.6.63:8090/api/khgz/project'

def fetch_project_info(name,url,dataSource):
    try:
        repeatUrl = 'http://10.220.6.138/api/khgz/project/getRepeat'
        header = {
            'Content-Type': 'application/json'
        }
        param = {
            'name': name,
            'url': url,
            'dataSource': dataSource
        }
        response = requests.post(url=repeatUrl,headers=header,json=param)
        data = json.loads(response.text)['data']
        return data
    except Exception as e:
        print(datetime.now(),'数据获取失败',e)
        return False

def insertProject(param):
    try:
        # print('插入数据',param['name'])
        header = {
            'Content-Type': 'application/json'
        }
        response = requests.post(url=insertUrl,headers=header,json=[param])
        code = json.loads(response.text)['code']
        if code == 0 :
            print(datetime.now(),'新增成功:',param.get('name'))
        else:
            msg = json.loads(response.text)['msg']
            print(datetime.now(),param.get('name'),'-新增失败:',msg)

    except Exception as e:
        print(datetime.now(),param.get('name'),'-数据新增异常错误:',e)


# -*- coding: UTF-8 -*-
"""
@File    ：dingTalkMsgHelper.py
@Author  ：VerSion/08398
@Date    ：2023/12/08 11:28 
@Corp    ：Uniview
"""

import json
from datetime import datetime
from noteparse.configReader import readConfig

import requests

dingConf = readConfig('ding_url','app_key','app_secret','agent_id','rebot_code','user_id_license',section='dingTalkConfig')

def get_token():
    """
    获取钉钉接口token
    :return: token字符串
    """
    try:
        url = f"{dingConf['ding_url']}/gettoken?appkey={dingConf['app_key']}&appsecret={dingConf['app_secret']}"
        response = requests.get(url)
        result = json.loads(response.text)
        if result['errcode'] == 0:
            return result['access_token']
        else:
            raise Exception(result['errmsg'])
    except Exception as ex:
        raise Exception('获取token失败：' + str(ex))


def get_user_id(user_no):
    """
    根据工号查询员工的钉钉userid
    :param user_no:员工工号（带首字母，例如Z00001）
    :return:传入员工的钉钉userid
    """
    url = "https://dip.uniview.com/api/interface/query/getUserIdByUserNo"
    request_headers = {
        "license": dingConf['user_id_license']
    }
    request_body = {
        "userNo": user_no.upper()
    }

    try:
        response = requests.post(url, headers=request_headers, json=request_body)
        if response.status_code == 200:
            data = response.json()
            if data["isSuccess"]:
                rtn_list = data["rtnList"]
                if rtn_list and len(rtn_list) > 0:
                    return rtn_list[0]["USERID"]
                else:
                    raise Exception(f'员工 {user_no} 不存在')
            else:
                raise Exception(data["rtnMsg"])
        else:
            raise Exception('接口调用失败，失败代码 ' + str(response.status_code))
    except Exception as ex:
        raise Exception('获取userid失败：' + str(ex))


def send_dingwork_message(access_token, userid_list, content):
    """
    向用户发送钉钉消息
    :param access_token:钉钉接口token
    :param userid_list:用户的userid列表字符串，多个用户间用英文逗号隔开
    :param content:消息正文
    :return:是否发送成功
    """
    try:
        url = f"{dingConf['ding_url']}/topapi/message/corpconversation/asyncsend_v2?access_token={access_token}"
        request_headers = {
            'Content-Type': 'application/json'
        }
        request_body = {
            'agent_id': dingConf['agent_id'],
            'userid_list': userid_list,
            'msg': {
                'msgtype': 'text',
                'text': {
                    'content': content
                }
            }
        }

        response = requests.post(url, headers=request_headers, json=request_body)
        if response.status_code == 200:
            data = response.json()
            errcode = data['errcode']
            errmsg = data['errmsg']
            if errcode == 0:
                return True
            else:
                raise Exception(errmsg)
        else:
            raise Exception('接口调用失败，失败代码 ' + str(response.status_code))
    except Exception as ex:
        raise Exception('发送钉钉消息失败：' + str(ex))


def send_dingwork_card(access_token, userid_list, title, content, button_url, button_title="查看详情"):
    """
    向用户发送钉钉卡片消息
    :param access_token:钉钉接口token
    :param userid_list:用户的userid列表字符串，多个用户间用英文逗号隔开
    :param title: 消息标题
    :param content:消息正文（支持markdown语法）
    :param button_url: 下方跳转按钮的链接
    :param button_title: 下方跳转按钮的标题
    :return:是否发送成功
    """
    try:
        url = f"{dingConf['ding_url']}/topapi/message/corpconversation/asyncsend_v2?access_token={access_token}"
        request_headers = {
            'Content-Type': 'application/json'
        }
        request_body = {
            'agent_id': dingConf['agent_id'],
            'userid_list': userid_list,
            'msg': {
                'msgtype': 'action_card',
                'action_card': {
                    'title': title,
                    'markdown': content,
                    'single_title': button_title,
                    'single_url': button_url
                }
            }
        }

        response = requests.post(url, headers=request_headers, json=request_body)
        if response.status_code == 200:
            data = response.json()
            errcode = data['errcode']
            errmsg = data['errmsg']
            if errcode == 0:
                return True
            else:
                raise Exception(errmsg)
        else:
            raise Exception('接口调用失败，失败代码 ' + str(response.status_code))
    except Exception as ex:
        raise Exception('发送钉钉消息失败：' + str(ex))


def send_dingwork_card_by_robot(access_token, userid_list, title, content, button_url, button_title="查看详情"):
    """
    通过机器人向用户发送钉钉卡片消息
    :param access_token:钉钉接口token
    :param userid_list:用户的userid列表字符串，多个用户间用英文逗号隔开
    :param title: 消息标题
    :param content:消息正文（支持markdown语法）
    :param button_url: 下方跳转按钮的链接
    :param button_title: 下方跳转按钮的标题
    :return:是否发送成功
    """
    try:
        url = f'https://api.dingtalk.com/v1.0/robot/oToMessages/batchSend'
        request_headers = {
            'Content-Type': 'application/json',
            'x-acs-dingtalk-access-token': access_token
        }
        msg_param = {
            'title':title,
            'text':content,
            'singleTitle':button_title,
            'singleURL':button_url
        }
        request_body = {
            'robotCode': dingConf['rebot_code'],
            'userIds': userid_list,
            'msgKey': 'sampleActionCard',
            'msgParam':json.dumps(msg_param, ensure_ascii=False)
        }

        response = requests.post(url, headers=request_headers, json=request_body)
        if response.status_code == 200:
            return True
        else:
            err_obj = response.json()
            errcode = err_obj['code']
            errmsg = err_obj['message']
            raise Exception(f'接口调用失败，失败代码[{errcode}]：{errmsg}')
    except Exception as ex:
        raise Exception('机器人发送钉钉消息失败：' + str(ex))


def send_msg_to_employee(employee_list, content):
    """
    根据工号发送钉钉消息
    :param employee_list: 字符串数组，内容为员工带首字母的工号。例如：['Z00001','X00002']
    :param content:
    :return:
    """
    try:
        if not isinstance(employee_list, list) or not all(isinstance(e, str) for e in employee_list):
            raise TypeError("传入参数employee_list必须是字符串数组")
        userid_list = ",".join([get_user_id(employee) for employee in employee_list])
        token = get_token()
        now = datetime.now()
        formatted_time = now.strftime("[%Y-%m-%d %H:%M:%S]")
        content_with_time = content + " " + formatted_time
        return send_dingwork_message(token, userid_list, content_with_time)
    except Exception as ex:
        raise ex


def send_card_to_employee(employee_list, title, content, button_url, button_title="查看详情"):
    """
    根据工号发送钉钉卡片消息
    :param employee_list: 字符串数组，内容为员工带首字母的工号。例如：['Z00001','X00002']
    :param title: 消息标题
    :param content:消息正文（支持markdown语法）
    :param button_url: 下方跳转按钮的链接
    :param button_title: 下方跳转按钮的标题
    :return:是否发送成功
    """
    try:
        if not isinstance(employee_list, list) or not all(isinstance(e, str) for e in employee_list):
            raise TypeError("传入参数employee_list必须是字符串数组")
        userid_list = ",".join([get_user_id(employee) for employee in employee_list])
        token = get_token()
        now = datetime.now()
        formatted_time = now.strftime("[%Y-%m-%d %H:%M:%S]")
        content_with_time = content + "<br>" + formatted_time
        return send_dingwork_card(token, userid_list, title, content_with_time, button_url, button_title)
    except Exception as ex:
        raise ex

def send_card_to_employee_by_robot(employee_list, title, content, button_url, button_title="查看详情"):
    """
    根据工号通过机器人发送钉钉卡片消息
    :param employee_list: 字符串数组，内容为员工带首字母的工号。例如：['Z00001','X00002']
    :param title: 消息标题
    :param content:消息正文（支持markdown语法）
    :param button_url: 下方跳转按钮的链接
    :param button_title: 下方跳转按钮的标题
    :return:是否发送成功
    """
    try:
        if not isinstance(employee_list, list) or not all(isinstance(e, str) for e in employee_list):
            raise TypeError("传入参数employee_list必须是字符串数组")
        userid_list = [get_user_id(employee) for employee in employee_list]
        token = get_token()
        return send_dingwork_card_by_robot(token, userid_list, title, content, button_url, button_title)
    except Exception as ex:
        raise ex


import setuptools  # 导入setuptools打包工具

# with open("README.md", "r", encoding="utf-8") as fh:
#     long_description = fh.read()

setuptools.setup(
    name="noteparse",  # 用自己的名替换其中的YOUR_USERNAME_
    version="1.1.4",  # 包版本号，便于维护版本,保证每次发布都是版本都是唯一的
    author="maoyuyan",  # 作者，可以写自己的姓名
    author_email="294567571@qq.com",  # 作者联系方式，可写自己的邮箱地址
    description="a package for parse html to noteinfo",  # 包的简述
    long_description=open("README.md", "r", encoding="utf-8").read(),  # 包的详细介绍，一般在README.md文件内
    long_description_content_type="text/markdown",
    platforms=["all"],
    # url="https://github.com/m294567571/noteparse.git",  # 自己项目地址，比如github的项目地址
    # http://gitlab.uniview.com/mkt_uic/uic-spider.git
    packages=setuptools.find_packages(),
    # entry_points={
    #     "console_scripts" : ['noteparse = noteparse.__init__:init']
    # }, #安装成功后，在命令行输入mwjApiTest 就相当于执行了mwjApiTest.manage.py中的run了
    install_requires=[
        'requests>=2.32.3',
        'bs4>=0.0.2',
        'fastapi>=0.115.4',
        'pydantic>=2.7.3',
        'loguru>=0.7.2',
        'zhipuai>=2.1.5.20230904',
        'selenium>=4.21.0',
        'fake-useragent>=1.5.1'
    ],
    classifiers=[
        # 发展时期,常见的如下
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',
        # 开发的目标用户
        'Intended Audience :: Developers',
        # 许可证信息
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft :: Windows',
        'Natural Language :: Chinese (Simplified)',
        # 目标 Python 版本
        'Programming Language :: Python :: 3',
        # 属于什么类型
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    license='MIT'
)



from ossHelper import upload_file_to_minio, generate_snowflake_id
from playwright.sync_api import sync_playwright
import os
from datetime import datetime
# url--网页路径，selector--加载完成的元素选择器
def screenShotByPlaywright(url,selector,prefix='DXYG'):
    with sync_playwright() as playwright:
        try:
            browser = playwright.chromium.launch(args=['--font-render-hinting=medium'],chromium_sandbox=False)
            page = browser.new_page()
            page.goto(url)
            
            # scroll_height = page.evaluate('document.body.scrollHeight')
            # page.set_viewport_size({"with":1920,"height":scroll_height})
            # 等待元素加载完成
            content = page.wait_for_selector(selector)
            html = page.content()
            screenshot_name = f'{prefix}-screenshot-' + generate_snowflake_id() + ".png"
            page.screenshot(path=screenshot_name,full_page=True)
            imgurl = upload_file_to_minio(screenshot_name, 'screenshot/' + screenshot_name)
            browser.close()
            # 截图完毕后删除截图
            os.remove(screenshot_name)
            return imgurl
        except Exception as screen_img_err:
            print(datetime.now(),f'网页截图失败，网址：{url}',screen_img_err)
            return ''
# 设定main函数,程序起点
if __name__ == '__main__':
    screenShotByPlaywright('https://caigou.chinatelecom.com.cn/DeclareDetails?id=7196160872105676&type=1&docTypeCode=TenderAnnouncement','.mbg')


from selenium import webdriver
from selenium.webdriver.common.by import By  
from selenium.webdriver.support.ui import WebDriverWait  
from selenium.webdriver.support import expected_conditions as EC  
import zipfile
import string
import time
from datetime import datetime
import noteparse.proxyHelper as proxy
import traceback
from fake_useragent import UserAgent
from noteparse.configReader import readConfig


proxyConf = readConfig('minute_url','minute_user','minute_pwd','daily_url','daily_user','daily_pwd',section='proxyConfig')

# 带账密的代理selenium
def create_proxy_auth_extension(proxy_host, proxy_port, proxy_username, proxy_password, scheme='http',
                                plugin_path=None):
    if plugin_path is None:
        plugin_path = r'{}_{}@http-dyn.dobel.com_9020.zip'.format(proxy_username, proxy_password)

    manifest_json = """
    {
        "version": "1.0.0",
        "manifest_version": 2,
        "name": "Dobel Proxy",
        "permissions": [
            "proxy",
            "tabs",
            "unlimitedStorage",
            "storage",
            "<all_urls>",
            "webRequest",
            "webRequestBlocking"
        ],
        "background": {
            "scripts": ["background.js"]
        },
        "minimum_chrome_version":"22.0.0"
    }
    """

    background_js = string.Template(
        """
        var config = {
            mode: "fixed_servers",
            rules: {
                singleProxy: {
                    scheme: "${scheme}",
                    host: "${host}",
                    port: parseInt(${port})
                },
                bypassList: ["foobar.com"]
            }
          };

        chrome.proxy.settings.set({value: config, scope: "regular"}, function() {});

        function callbackFn(details) {
            return {
                authCredentials: {
                    username: "${username}",
                    password: "${password}"
                }
            };
        }

        chrome.webRequest.onAuthRequired.addListener(
            callbackFn,
            {urls: ["<all_urls>"]},
            ['blocking']
        );
        """
    ).substitute(
        host=proxy_host,
        port=proxy_port,
        username=proxy_username,
        password=proxy_password,
        scheme=scheme,
    )

    with zipfile.ZipFile(plugin_path, 'w') as zp:
        zp.writestr("manifest.json", manifest_json)
        zp.writestr("background.js", background_js)

    return plugin_path


def from_proxy_get_daili(proxy):
    # proxy是这种格式 user:pass@ip:port
    user_pass_str, ip_port_str = proxy.split('@')
    proxyHost, proxyPort = ip_port_str.split(':')
    proxyUser, proxyPass = user_pass_str.split(':')
    return proxyHost, proxyPort, proxyUser, proxyPass

def getDriver(proxyInfo):
    proxyHost, proxyPort, proxyUser, proxyPass = from_proxy_get_daili(proxyInfo)
    proxy_auth_plugin_path = create_proxy_auth_extension(
        proxy_host=proxyHost,
        proxy_port=proxyPort,
        proxy_username=proxyUser,
        proxy_password=proxyPass)
    option = webdriver.ChromeOptions()
    option.add_argument('--headless=new') #静默模式,不打开浏览器
    option.add_argument('--no-sandbox')
    option.add_argument('--disable-dev-shm-usage')
    option.add_argument('--disable-gpu')
    option.add_argument('log-level=3')
    option.add_extension(proxy_auth_plugin_path)
    # current_location = os.path.dirname(__file__)
    # path = os.path.join(current_location,"BidFile")
    # prefs = { 'profile.default_content_settings.popups': 0, 'download.default_directory': path}
    # option.add_experimental_option( 'prefs', prefs)
    # proxyInfo = proxy.get_daily_ip()
    # proxyIp = "http://imdlxin:imdlxin@"+str(proxyInfo['ip'])+":"+str(proxyInfo['port'])
    # print('--daili--:',proxyIp)
    # option.add_argument(f'--proxy-server={proxyIp}')
    # 放弃样式\图像\子帧
    driver = webdriver.Chrome(options=option)
    return driver

def get_driver():
    proxyInfo = proxy.get_daily_ip()
    ip = proxyInfo['ip']
    port = proxyInfo['port']
    proxys = f"{proxyConf['daily_user']}:{proxyConf['daily_pwd']}@{ip}:{port}"
    driver = getDriver(proxys)
    return driver

def getCookie():
    print(datetime.now(),'==========开始获取cookie======')
    try:
        proxyInfo = proxy.get_daily_ip()
        ip = proxyInfo['ip']
        port = proxyInfo['port']
        proxys = f"{proxyConf['daily_user']}:{proxyConf['daily_pwd']}@{ip}:{port}"
        driver = getDriver(proxys)
        # 打开登录页
        loginUrl = 'http://qiye.qianlima.com/yfbsite/a/login'
        driver.get(loginUrl)
        WebDriverWait(driver, 60).until(
            EC.presence_of_element_located((By.XPATH, '//*[@id="loginTypeForm"]'))
        )
        pwdLoginBtn = driver.find_element(By.XPATH,'//*[@id="accountLogin"]/i')
        pwdLoginBtn.click()
        time.sleep(2)
        userInput = driver.find_element(By.XPATH,'//*[@id="username"]')
        pwdInput = driver.find_element(By.XPATH,'//*[@id="password"]')
        # userInput.send_keys(username)
        # pwdInput.send_keys(username)
        # 等待登录
        # isLogin = False
        # while not isLogin:
        #     try:
        #         qrCode = driver.find_element(By.ID, 'qrcode')
        #         if qrCode:
        #             print(datetime.now(),"未登录")
        #             time.sleep(2)
        #         else:
        #             isLogin = True
        #     except Exception:
        #         isLogin = True

        print(datetime.now(),'登录完成!')

        jessionId = driver.get_cookie('yfbSite.session.id')
        cookie = jessionId.get('value')
        # header['Cookie'] = f'yfbSite.session.id={cookie}'
        print(datetime.now(),'jessionid',cookie)
    except Exception as get_cookie_err:
        print(datetime.now(),'获取cookie失败',get_cookie_err)
        traceback.print_exc()